﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Steema.TeeChart.Styles;
using System.Threading;
using Steema.TeeChart;

namespace TestMemoryLeakageOfTeechart
{
    public partial class Form1 : Form
    {
        Random objRandom = new Random();
        DataTable dtData = new DataTable();
        List<FastLine> lstFSeries = new List<FastLine>();
        List<TChart> lstTChartCollection = new List<TChart>();
        const int MAXSERIES = 20;
        const int NOOFPOINTS = 600;
        const int CHARTHEIGHT = 300;
        TableLayoutPanel tblChartPanel = new TableLayoutPanel();
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            GenerateCharts();            

            GetDataTableStructure();

            GenerateSeries();
            tblChartPanel.Dock = DockStyle.Fill;
            tblChartPanel.AutoScroll = true;
            this.Controls.Add(tblChartPanel);
            Thread thdMain = new Thread(new ThreadStart(ThreadCallback));
            thdMain.IsBackground = true;
            thdMain.Start();
        }

        private void GenerateCharts()
        {
            tblChartPanel.Height = CHARTHEIGHT * MAXSERIES;
            for (int j = 0; j < MAXSERIES; j++)
            {
                //tblChartPanel.RowStyles.Add(new RowStyle(SizeType.AutoSize));
                TChart tChart = new TChart();
                InitializeTeeChart(tChart);                
                tblChartPanel.Controls.Add(tChart);
                lstTChartCollection.Add(tChart);
            }
            
        }

        private void InitializeTeeChart(TChart tChart)
        {
            tChart.Aspect.View3D = false;
            tChart.Legend.Visible = false;
            tChart.Height = CHARTHEIGHT;
            tChart.Dock = DockStyle.Fill;
        }

        private void GenerateSeries()
        {
            for (int j = 0; j < MAXSERIES; j++)
            {
                FastLine fs = new FastLine(lstTChartCollection[j].Chart);                
                lstFSeries.Add(fs);
            }
        }

        private void GetDataTableStructure()
        {
            for (int i = 0; i < MAXSERIES; i++)
            {
                dtData.Columns.Add("Column" + i.ToString("00"));
            }
            dtData.Columns.Add("DateTime");
        }

        private void ThreadCallback()
        {
            while (true)
            {
                dtData.Rows.Clear();
                for (int i = 0; i < NOOFPOINTS; i++)
                {
                    DataRow dr = dtData.NewRow();
                    for (int j = 0; j < MAXSERIES; j++)
                    {
                        dr["Column" + j.ToString("00")] = GetRandomValue();
                    }

                    dr["DateTime"] = DateTime.Now.Subtract(new TimeSpan((NOOFPOINTS - i) * 1000)).ToString("yyyy-MM-dd") + "\n" + DateTime.Now.Subtract(new TimeSpan((NOOFPOINTS - i) * 1000)).ToString("HH:mm:ss.fff");
                    dtData.Rows.Add(dr);
                }
                this.Invoke(new EventHandler(delegate { UpdateDataToChart(dtData); }));
                Thread.Sleep(1000);
            }
        }

        private void UpdateDataToChart(DataTable dtChartData)
        {
            for (int i = 0; i < MAXSERIES; i++)
            {
                lstFSeries[i].Clear();
                lstFSeries[i].Labels.Clear();
                lstFSeries[i].DataSource = dtChartData;
                lstFSeries[i].Color = Color.Green;
                lstFSeries[i].LabelMember = "DateTime";
                lstFSeries[i].YValues.DataMember = dtChartData.Columns[i].ColumnName;
                lstFSeries[i].CheckDataSource();
            }
        }

        private double GetRandomValue()
        {
            return objRandom.NextDouble();
        }
    }
}
